<?php
class plgHikashoppaymentAmazonCheckout extends hikashopPaymentPlugin  {
	
	var $accepted_currencies = array('USD');
	var $multiple = true;
	var $name = 'amazoncheckout';
	var $doc_form = 'amazoncheckout';
	var $button = '';
	
	//plugin config & input : 
	var $pluginConfig = array(
		
		'MerchantId' => array ('Merchant Id', 'input'),
		
		//required to sign the XMLcart
		'AccessKeyID' => array ('AWS Access Key ID', 'input'),
		'SecretAccessKey' => array ('Secret Access Key', 'input'),
		'displaycheckout' => array('DISPLAY_BUTTON_CHECKOUT', 'boolean','0'),
		'displaycart' => array('DISPLAY_BUTTON_CART', 'boolean','0'),
		'readytoship' => array('Order status upon ready to ship notification', 'orderstatus'),
		'debug' => array('DEBUG', 'boolean','0'),
		'environnement' => array('ENVIRONNEMENT', 'list',array(
			'production' => 'HIKA_PRODUCTION',
			'sandbox' => 'HIKA_SANDBOX')
		),
		'invalid_status' => array('INVALID_STATUS', 'orderstatus'),
		'verified_status' => array('VERIFIED_STATUS', 'orderstatus'),
		'return_url' => array('RETURN_URL', 'html'),
		
		//Button option : 
		'html1' => array('For Amazon button style','html',''),
		'size' => array('Size', 'list',array(
			'large' => 'large',
			'medium' => 'medium',
			'x-large' => 'x-large'
		)),
		'color' => array('Color', 'list',array(
			'orange' => 'orange',
			'tan' => 'tan'
		)),
		'background'=> array('Background', 'list',array(
			'white' => 'white',
			'light' => 'light',
			'dark' => 'dark'
		)),
	);
	
	function __construct(&$subject, $config) {
		//Set return url :
		$this->pluginConfig['return_url'][2] = htmlentities(HIKASHOP_LIVE . '/index.php?option=com_hikashop&ctrl=checkout&task=notify&notif_payment=amazoncheckout');
		
		return parent::__construct($subject, $config);
	}
	
	function onPaymentDisplay(&$order,&$methods,&$usable_methods) {
		//Keep empty to not display on payment methods listing
	}

	function onAfterOrderConfirm(&$order,&$methods,$method_id) {
		parent::onAfterOrderConfirm($order,$methods,$method_id);
		
	}

	//To set the back end default values	
	function getPaymentDefaultValues(&$element) {
		$element->payment_name='amazon checkout';
		$element->payment_description='You can pay by credit card using this payment method';
		$element->payment_images='MasterCard,VISA,Credit_card,American_Express';
		$element->payment_params->notification=1;
		$element->payment_params->invalid_status='cancelled';
		$element->payment_params->verified_status='confirmed';
	}

	function onPaymentNotification(&$statuses) {
		
		//======================================================================
		//  Instant ordering process :
		//======================================================================
		
		// Get amazon notification data :
		$notifType = JRequest::getVar('NotificationType');
		$notifData = $_POST["NotificationData"];
		$notifUuid = JRequest::getVar("UUID");
		$notifTimestp = JRequest::getVar("Timestamp");
		$notifSign = JRequest::getVar("Signature");
		$notifAwsKeyId = JRequest::getVar("AWSAccessKeyId");
		
		// Check : Hikashop work only with Amazon signed request :
		if (empty($notifSign) ) {
			
			$this->debugcheck('Error! \r\n', 'Hikashop only works with Amazon signed request, you must define amazon seller central settings');
			header('HTTP/1.1 500 Internal Server Error');
			exit;
		}
		// Check : Data missing?...
		if (empty($notifData) ) {
			
			$this->debugcheck('Error! \r\n', 'Missing data in Amazon Notification');
			header('HTTP/1.1 500 Internal Server Error');
			exit;
		}
		
		// Check notification data :
		$postarray = print_r($_POST,true);
		$this->debugcheck('POST : ', htmlentities($postarray) );
		
		// Load payment params :
		$orderobject = simplexml_load_string($notifData);
		
		// get payment id from Amazon notification CartCustomData : 
		$payment_id = (string)$orderobject->ProcessedOrder->ProcessedOrderItems->ProcessedOrderItem->CartCustomData->PaymentNumber;
		
		// Set it in plugin params : 
		$this->pluginParams($payment_id);
		$this->payment_params =& $this->plugin_params;
		
		if(empty($this->payment_params)) {
			$this->debugcheck("Impossible to find plugins params with id : ", $payment_id);
			header('HTTP/1.1 500 Internal Server Error');
			exit;
		}
		
		// Notification check process :
		//============================================================================================
		// Access Key Id match ? :
		if ($this->plugin_params->AccessKeyID != $notifAwsKeyId) {
			$errdata = $this->plugin_params->AccessKeyID . " != " . $notifAwsKeyId;
			$this->debugcheck("notification Access key id mistmatch ! : \r\n ", $errdata);
			header('HTTP/1.1 403 Forbidden');
			exit;
		}
		
		// Sign match ? : 
		$strsign = $notifUuid . $notifTimestp;
		$sign = $this->createsign("sha1", $strsign, $this->plugin_params->SecretAccessKey);
		
		if ($sign != $notifSign) {
			$errdata = $sign . " != " . $notifSign;
			$this->debugcheck("notification Signature mistmatch ! : \r\n", $errdata);
			header('HTTP/1.1 403 Forbidden');
			exit;
		}
		
		// (end check process) ==========================================================================
		// 
		// Check if order have been already created thanks to the "AmazonOrderItemCode" : 
		$AmazonNumber = (string)$orderobject->ProcessedOrder->ProcessedOrderItems->ProcessedOrderItem->AmazonOrderItemCode;
		$db = JFactory::getDBO();
		$query = 'SELECT order_id FROM ' . hikashop_table('order') .
			' WHERE `order_payment_params` LIKE \'%'.hikashop_getEscaped($AmazonNumber,true).'%\'';
		$db->setQuery($query);
		$order_id = $db->loadResult();
		
		// View on POST/Amazon Notification : 
		$this->debugcheck("NotificationData = ", htmlentities($notifData) );
		
		// New order creation :
		if ($notifType == 'NewOrderNotification') {	
			
			// Is this check required?
			/* 
			 *  Already check existing order thanks to Amazon Number later :
			 *  And can lead to trouble if different time on the servers.
			 * 
			// Time stamp delay (not more 15 min delay granted): ONLY FOR NEWORDERNOTIFICATION
			$unix = time();
			$notifunix = strtotime($notifTimestp);

			if ($unix - $notifunix >= 900) {
				$this->debugcheck("time delay too long ! : \r\n", $unix - $notifunix);
				header('HTTP/1.1 403 Forbidden');
				exit;
			}
			 */
			
			// At least one order have already this Amazon Number...
			if ($order_id) {
				$this->debugcheck('order amazon number already found : ', $AmazonNumber);
				$this->debugcheck('Already created order order id ref : ', $order_id);
				
				//Response to Amazon:
				header('HTTP/1.1 200 OK');
				exit;
			}
			
			// Debug receive data :
			$this->debugcheck("NotificationType = ", $notifType );
			
			// Create user :
			$userid = $this->createUser((string)$orderobject->ProcessedOrder->BuyerInfo->BuyerEmailAddress);
			$this->debugcheck('user id : ',$userid);
			
			// Create Address :
			$addressid = $this->createAddress($orderobject,$userid);
			$this->debugcheck('address id : ',$addressid);
			
			// Create new order :
			$order = $this->createOrder ($orderobject, $userid, $addressid);
			
			// Check the order id :
			$this->debugcheck('order id : ', $order->order_id);
		
			//Response to Amazon:
			header('HTTP/1.1 200 OK');
			exit;
		}
		
		if ($notifType == 'OrderReadyToShipNotification') {
			
			//Debug receive data :
			$this->debugcheck("NotificationType = ", $notifType );
		
			//Order find thanks to amazon number?
			if (empty($order_id) ) {
				
				$this->debugcheck("No order match with Amazon number ready to ship notification : \n\r", $AmazonNumber);
				$this->debugcheck("NotificationData = ", htmlentities($notifData) );
				header('HTTP/1.1 200 OK');
				exit;
			}
			
			// Set order update :
			$update = new stdClass();
			$update->order_id = $order_id;
			$update->history = new stdClass();
			$update->history->history_type = 'update';
			
			// Update order to shipping status :
			$status = null;
			$statuschange = 'STATUS_NOT_CHANGED';
			
			if ($this->plugin_params->readytoship != '0') {
				// Update order status -> shipped OR confirmed
				$status = $this->plugin_params->readytoship;
				$statuschange = 'STATUS_CHANGED';
			}
			
			// Notified the customer : 
			$notified = 1;
			
			// Get order params : :
			$dbOrder = $this->getOrder($order_id);
			
			// Prepare email for merchant:
			$url = HIKASHOP_LIVE.'administrator/index.php?option=com_hikashop&ctrl=order&task=edit&order_id='.$order_id;
			$details = 'Ready to ship notification from Amazon recieved';
			$order_text = "\r\n".JText::sprintf('NOTIFICATION_OF_ORDER_ON_WEBSITE',$dbOrder->order_number,HIKASHOP_LIVE);
			$order_text .= "\r\n".str_replace('<br/>',"\r\n",JText::sprintf('ACCESS_ORDER_WITH_LINK',$url));

			$email = new stdClass();
			$email->subject = JText::sprintf('PAYMENT_NOTIFICATION_FOR_ORDER','Atos Sips',$status,$dbOrder->order_number);
			$body = str_replace('<br/>',"\r\n",JText::sprintf('PAYMENT_NOTIFICATION_STATUS','Atos Sips',$status)).' '.JText::_($statuschange)
			."\r\n" . $details . "\r\n".$order_text;
			$email->body = $body;

			// History get order data:
			$history = new stdClass();
			$history->notified = $notified;
			$history->data = $details;

			// Update order : 
			$this->modifyOrder($order_id, $status, $history, $email);
			
			// Response to Amazon :
			header('HTTP/1.1 200 OK');
			exit;
		}
		
		if ($notifType == 'OrderCancelledNotification') {
			
			// Debug receive data :
			$this->debugcheck("NotificationType = ", $notifType );
			
			// Order find ?
			if (empty($order_id) ) {
				
				$this->debugcheck("No order match with Amazon ready to ship notification : \n\r", $AmazonNumber);
				$this->debugcheck("NotificationData = ", htmlentities($notifData) );
				header('HTTP/1.1 200 OK');
				exit;
			}
			
			// Notified the customer : 
			$notified = 1;
			
			// Get order params :
			$dbOrder = $this->getOrder($order_id);
			
			// Set Canceled status :
			$status = $this->plugin_params->invalid_status;
			$statuschange = 'STATUS_CHANGED';
			
			//Prepare email for merchant:
			$url = HIKASHOP_LIVE.'administrator/index.php?option=com_hikashop&ctrl=order&task=edit&order_id='.$order_id;
			$details = 'Canceled notification from Amazon recieved';
			$order_text = "\r\n".JText::sprintf('NOTIFICATION_OF_ORDER_ON_WEBSITE',$dbOrder->order_number,HIKASHOP_LIVE);
			$order_text .= "\r\n".str_replace('<br/>',"\r\n",JText::sprintf('ACCESS_ORDER_WITH_LINK',$url));

			$email = new stdClass();
			$email->subject = JText::sprintf('PAYMENT_NOTIFICATION_FOR_ORDER','Amazon Checkout',$status,$dbOrder->order_number);
			$body = str_replace('<br/>',"\r\n",JText::sprintf('PAYMENT_NOTIFICATION_STATUS','Amazon Checkout',$status)).' '.JText::_($statuschange)
			."\r\n" . $details . "\r\n".$order_text;
			$email->body = $body;

			//history get order data:
			$history = new stdClass();
			$history->notified = $notified;
			$history->data = $details;

			//Update order : 
			$this->modifyOrder($order_id, $status, $history, $email);
			
			//Response to Amazon:
			header('HTTP/1.1 200 OK');
			exit;
		}
		
		// Log check :
		$this->debugcheck("Unknow notification from Amazon", ($notifType) );
		
		// No Notification Type recognize : 
		echo'HTTP/1.1 500 Internal Server Error';
		exit;
		// Instant ordering process : (END !!!)
		//======================================================================
	}
	
	function onHikashopBeforeDisplayView(&$element) {
		
		$this->pluginParams();
		$this->layout =  $element->getLayout();
		if (isset($element->ctrl))
			if($element->ctrl=='checkout' && $this->layout=='step' && $this->canDisplayButton($element))
				ob_start();
	}

	function onHikashopAfterDisplayView(&$element) {
		
		if (isset($element->ctrl)) {
			
			if ($element->ctrl=='product' && $this->layout=='cart' && $element->element->cart_type=='cart' && $this->canDisplayButton($element,'cart')) {
				
				//Set Xml cart :
				$XMLcart = $this->createXml($element);
				
				//log file :
				$this->debugcheck("XML create in Amazon checkout button :",htmlentities($XMLcart));
			
				$button = $this->createbutton($XMLcart);
				echo $button;
			}
			
			elseif($element->ctrl=='checkout' && $this->layout=='step' && $this->canDisplayButton($element)) {
				
				$contenttable = array();
				$contentth = array();
				$inserthtml = '<tr>';
				$html = ob_get_clean();

				if (preg_match_all('#<div id="hikashop_checkout_cart"(.*?)</table>#iUs',$html,$contenttable)) {
					//Set Xml cart :
					$XMLcart = $this->createXml($element);
					$button = $this->createbutton($XMLcart);
					$old_cart = $contenttable[0][0];
					preg_match_all('#</th>#i',$old_cart,$contentth);
					
					for ($i=0 ; $i<count($contentth[0])-2 ; $i++) //2 dynamic
						$inserthtml .= '<td></td>';
					$inserthtml .= '<td colspan="2">'.$button.'</td></tr></tbody>';
					$new_cart = str_replace('</tbody>',$inserthtml,$old_cart);
					$html = str_replace($old_cart,$new_cart,$html);
				}
				echo $html;
			}
		}
	}

	function canDisplayButton(&$view,$type='checkout') {
		
		static $method = null;
		if(is_null($method)) {
			
			$class = hikashop_get('class.cart');
			$cart = $class->loadFullCart(true);
			$methods = $this->loadPaymentMethod('','all',$cart);
			if($methods) {
				$already = array();
				$max = 0;
				foreach($methods as $k => $method) {
					if(!empty($method->payment_params)) {
						$methods[$k]->payment_params = @unserialize($method->payment_params);
					}
					$methods[$k]->enabled = true;
					if(empty($method->ordering)) {
						$max++;
						$methods[$k]->ordering = $max;
					}
					while(isset($already[$methods[$k]->ordering])) {
						$max++;
						$methods[$k]->ordering = $max;
					}
					$already[$methods[$k]->ordering] = true;
				}
				$usable_methods = array();
				parent::onPaymentDisplay($cart,$methods,$usable_methods);
				if(count($usable_methods)) {
					$method = reset($usable_methods);
				}
			}
		}
		
		if(is_object($method) && empty($method->errors)) {
			if ($method->payment_params->displaycart && $type=='cart')
				return true;
			elseif ($method->payment_params->displaycheckout && $type=='checkout')
				return true;
		}
		return false;
	}

	private function getCurrencyCode($currency_id){
		$currency_code = '';
		if($currency_id){
			$class = hikashop_get('class.currency');
			$dbCurrencyData = $class->get($currency_id);
			if($dbCurrencyData){
				$currency_code = $dbCurrencyData->currency_code;
			}
		}
		return $currency_code;
	}
	
	function debugcheck ($message,$data) {
		// Works if debug set on "Yes" : 
		if (!isset($this->plugin_params->debug) || $this->plugin_params->debug) {
			$this->writeToLog($message . " \n\r\n\r :");
			$this->writeToLog($data . " \n\r\n\r :");
		}
	}

	function createsign ($sha,$data,$secretkey) {
		
		// Compute the hmac on input data bytes, make sure to set returning raw hmac to be true
		$rawHmac = hash_hmac($sha, $data, $secretkey, true);

		// base64-encode the raw hmac
		$sign = base64_encode($rawHmac);
		
		return ($sign);
		
	}
	
	function createXml ($data) {
		
		//get currency code :
		$currency_id = hikashop_getCurrency();
		$currency = $this->getCurrencyCode($currency_id);

		//==================================================================
		//Set Xml cart :
		//==================================================================
		$XMLcart = '<?xml version="1.0" encoding="UTF-8"?>'.
		'<Order xmlns="http://payments.amazon.com/checkout/2009-05-15/">'.
			'<Cart>'. 
				'<Items>';
		
		$items = '';
		
		//Set items in XML cart :
		if (isset($data->full_cart) ) {
			$productlist = $data->full_cart->products; 
		} 
		if (isset($data->element) ) { 
			$productlist = $data->element->products;
		}
		if (isset($data->orderInfos) ) {
			$productlist = $data->orderInfos->products;
		}
		if (isset($data->rows) ) {
			$productlist = $data->rows;
		}
		
		foreach ($productlist as $product) {
			if((int)$product->cart_product_quantity <= 0)
				continue;
			
			$items .= '<Item> 
				<SKU>' . $product->product_id . '</SKU>
				<MerchantId>' . $this->plugin_params->MerchantId . '</MerchantId>
				<Title>' . $product->product_name . '</Title>
				<Price>
					<Amount>' . ($product->prices[0]->price_value)  .  '</Amount>
					<CurrencyCode>' . $currency . '</CurrencyCode>
				</Price>
				<Quantity>' . $product->cart_product_quantity . '</Quantity>
				<ItemCustomData>
					<CustomText>
						<cart_product_id>' . $product->cart_product_id . '</cart_product_id>
						<cart_product_option_parent_id>' . $product->cart_product_option_parent_id . '</cart_product_option_parent_id>
						<order_product_code>' . $product->product_code . '</order_product_code>
						<order_product_wishlist_id>' . $product->cart_product_wishlist_id . '</order_product_wishlist_id>
					</CustomText>
				</ItemCustomData>
			</Item>';
		}
	
		$XMLcart .=	$items . '
			</Items>
				<CartCustomData>
					<PaymentNumber>' . $this->plugin_data->payment_id . '</PaymentNumber>';
		
		if (isset($data->full_cart) ) {
			$XMLcart .='
					<TelephonCustomer>' . $data->full_cart->shipping_address->address_telephone . '</TelephonCustomer>
					<TelephonCustomer2>' . $data->full_cart->shipping_address->address_telephone2 . '</TelephonCustomer2>';
		}
		
		$XMLcart .='
				</CartCustomData>
			</Cart>
		</Order>';
		
		return ($XMLcart);
	}

	function createbutton ($XMLcart) {
		
		//encoded Xml :
		$encodedXMLcart = base64_encode($XMLcart);
		
		// compute the hmac on input data bytes, make sure to set returning raw hmac to be true
		$sign = $this->createsign ("sha1",$XMLcart,$this->plugin_params->SecretAccessKey);
		
		//set value (encodedXMLcart + signature and AWS access key id) :
		$orderInputValue = "type:merchant-signed-order/aws-accesskey/1;order:" . $encodedXMLcart .
			";signature:" . $sign . ";aws-access-key-id:" . $this->plugin_params->AccessKeyID;
		
		// Get button setting from payment params :
		$params = array(
			'merchantId' => $this->plugin_params->MerchantId,
			'buttonSettings' => array(
				'size' => $this->plugin_params->size,
				'color' => $this->plugin_params->color,
				'background' => $this->plugin_params->background
			),
			'orderInput' => array(
				'format' => 'XML',
				'value' => $orderInputValue
			)
		);
		$p = array();
		foreach($params['buttonSettings'] as $k => $v) {
			$p[] = $k . '=' . urlencode($v);
		}
		$p = implode('&', $p);
		if(!empty($p))
			$p = '&' . $p;
		
		// Define url for Amazon request form : 
		if ($this->plugin_params->environnement == 'sandbox') {
			// Sandbox test :
			$amazonUrl = "<script type='text/javascript' src='https://static-na.payments-amazon.com/cba/js/us/sandbox/PaymentWidgets.js'></script>";
			$imgbutton = '<img src="https://payments-sandbox.amazon.com/gp/cba/button?type=checkout&cartOwnerId='.$this->plugin_params->MerchantId.$p.'" style="cursor: pointer;"/>';
		} else {
			// Production phase : -->
			$amazonUrl = "<script type='text/javascript' src='https://static-na.payments-amazon.com/cba/js/us/PaymentWidgets.js'></script>";
			$imgbutton = '<img src="https://payments.amazon.com/gp/cba/button?type=checkout&cartOwnerId='.$this->plugin_params->MerchantId.$p.'" style="cursor: pointer;"/>';
		}
				
		// Required js for notify amazon :
		$button = $amazonUrl . '<div id="cbaButton">'.$imgbutton.'</div>'.
'<script type="text/javascript">
	new CBA.Widgets.StandardCheckoutWidget({
		merchantId:\''.$this->plugin_params->MerchantId.'\',
		orderInput:{
			format:\'XML\',
			value:\''.$orderInputValue.'\'
		}
	}).render("cbaButton");
</script>';
		
		return ($button);
	}
	
	function loadPaymentMethod($name,$type,&$cart) {
		
		static $datas = array();
		if(empty($name)){
			$name = $this->name;
		}
		if(!isset($datas[$name])){
			$db = JFactory::getDBO();
			$where = array('payment_type = '.$db->Quote($name),'payment_published=\'1\'');

			$shipping = '';
			if(!empty($cart->shipping))
				$shipping = $cart->shipping[0]->shipping_type.'_'.$cart->shipping[0]->shipping_id;
			if(!empty($shipping)){
				$where[] = '(payment_shipping_methods IN (\'\',\'_\') OR payment_shipping_methods LIKE \'%\n'
				.$shipping.'\n%\' OR payment_shipping_methods LIKE \''.$shipping.'\n%\' OR payment_shipping_methods LIKE \'%\n'
				.$shipping.'\' OR payment_shipping_methods LIKE \''.$shipping.'\')';
			}
			$currency = hikashop_getCurrency();
			if(!empty($currency)){
				$where[] = "(payment_currency IN ('','_','all') OR payment_currency LIKE '%,".intval($currency).",%')";
			}

			$app = JFactory::getApplication();
			if(!$app->isAdmin()){
				hikashop_addACLFilters($where,'payment_access');
			}

			if(!empty($where)) {
				$where = ' WHERE '.implode(' AND ',$where);
			} else {
				$where = '';
			}

			$sql = 'SELECT * FROM `#__hikashop_payment`'.$where.' ORDER BY payment_ordering';
			$db->setQuery($sql);
			$datas[$name] = $db->loadObjectList();
		}

		if (!empty($datas[$name])){
			if($type=='id'){
				return $datas[$name][0]->payment_id;
			}elseif($type=='first'){
				return $datas[$name][0];
			}else{
				return $datas[$name];
			}
		}

		return false;
	}
	
	function createOrder($orderobject, $userid, $addressid) {
		
		// Set components product, shipping, tax from object order :
		$products = array();
		$i = 0;
		$tax = 0;
		$shippingPrice = 0;
		$shippingTax = 0;
		$total = 0;
		foreach ($orderobject->ProcessedOrder->ProcessedOrderItems->ProcessedOrderItem as $item) {
			
			$object = new stdClass();
			$object->product_id = (string)$item->ItemCustomData->CustomText->product_id;
			$object->order_product_quantity = (string)$item->Quantity;
			$object->order_product_name = (string)$item->Title;
			$object->order_product_code = (string)$item->ItemCustomData->CustomText->order_product_code;

			$component = array();
			
			// Price details :
			foreach ($item->ItemCharges->Component as $data) {
				$component [(string)$data->Type]= (string)$data->Charge->Amount;
			}
			
			$object->order_product_tax = $component['Tax'];
			$object->order_product_tax_info = '';
			$object->order_product_price = $component['Principal'] - $component['PrincipalPromo'];
			// Tax total
			$totaltax = '';
			$totaltax += $component['Tax'] + $component['ShippingTax'];
			$object->order_product_total_price_no_vat = $object->order_product_price * $object->order_product_quantity;
			$object->order_product_total_price = ($object->order_product_total_price_no_vat + $object->order_product_tax) * $object->order_product_quantity;

			$products[$i] = $object;
			$i +=1;
			$shippingPrice += $component['Shipping'] + $component['ShippingTax'] - $component['ShippingPromo'];
			$shippingTax +=  $component['ShippingTax'];
			//order total :
			$total += $object->order_product_total_price + $shippingPrice;
		}
		
		$object->order_tax_info = $tax;
		$object->order_product_total_price_no_vat = $total - $tax;
		
		//Set tax array: 
		$taxinfo = new stdClass();
		$taxinfo->tax_namekey = '';
		$taxinfo->tax_rate = 0.00;
		$taxinfo->tax_amount = $totaltax;
		$taxarray = array ($taxinfo);
		
		// Load Cart : 
		$cartClass = hikashop_get('class.cart');
		$cart = $cartClass->loadFullCart(true);
		
		// Load data from Hikashop BD :
		$config =& hikashop_config();
		
		// Get currency code from order object :
		$currencyCode = (string)$orderobject->ProcessedOrder->ProcessedOrderItems->ProcessedOrderItem->Price->CurrencyCode;
		$db = JFactory::getDBO();
		$sql ="SELECT currency_id FROM `#__hikashop_currency` WHERE currency_code = " . $db->Quote($currencyCode);
		$db->setQuery($sql);
		$currencyId = $db->loadResult();
	
		$AmazonNumber = (string) $orderobject->ProcessedOrder->ProcessedOrderItems->ProcessedOrderItem->AmazonOrderItemCode;
		
		// CREATE NEW ORDER : 
		$order = new stdClass();
		$order->order_user_id = $userid;
		$order->order_status = $config->get('order_confirmed_status');
		$order->order_shipping_address_id = $addressid;
		$order->order_billing_address_id = $addressid;
		$order->order_discount_code = '';
		$order->order_currency_id = $currencyId;
		$order->order_type = 'sale';
		$order->order_full_price = $total;
		$order->order_tax_info = $taxarray;

		$cart->products = $products;
		$order->order_shipping_price = $shippingPrice;
		$order->order_shipping_tax = $shippingTax;
		$order->order_shipping_params = null;
		$order->order_discount_tax = '';
		$order->order_discount_price = '';
		$order->cart =& $cart;
		$order->history = new stdClass();
		$order->history->history_reason = JText::_('ORDER_CREATED');
		$order->history->history_notified = 0;
		$order->history->history_type = 'creation';
		
		// Set order :
		$orderClass = hikashop_get('class.order');
		$order->order_payment_id = $this->plugin_data->payment_id;
		$order->order_payment_method = $this->name;
		$order->order_payment_params = new stdClass();
		$order->order_payment_params->amazon_number = $AmazonNumber;

		// SAVE
		$order->order_id = $orderClass->save($order);
	
		return $order;
	}
	
	function createUser($email) {
		
		$userClass = hikashop_get('class.user');
		
		// Check if user email already exist :
		$checkUser = $userClass->get($email, 'email', false);
		
		//Get user id (existing) OR creating a new user
		if(isset($checkUser) ) {
			// Get existing user id
			$userid = $checkUser->user_id;
		} 
		else {
			// New email user then get user_id:
			$user = new stdClass();
			$user->user_cms_id = 0;
			$user->user_email = $email;
			$userid = $userClass->save($user);
		}
		
		return $userid;
	}

	function createAddress($orderobject,$userid) {
		
		if(empty($userid) )
			return false;
			
		$db = JFactory::getDBO();
		$sql ="SELECT zone_name_english FROM `#__hikashop_zone` hz WHERE zone_type='country' AND zone_code_2 = " . $db->Quote((string)$orderobject->ProcessedOrder->ShippingAddress->CountryCode);
		$db->setQuery($sql);
		$country = $db->loadResult();

		$state = (string)$orderobject->ProcessedOrder->ShippingAddress->CountryCode;

		//Set Name in user Hikashop DB :
		$name = (string)$orderobject->ProcessedOrder->ShippingAddress->Name;
		$array = explode (' ', $name);
		$count = count ( $array );

		switch ($count) :
			case 1:
				$firstname = '';
				$lastname = $name;
				break;
			case 2:
				$firstname = $array[0];
				$lastname = $array[1];
				break;
			case 3: 
				$firstname = $array[0] . " " . $array[1];
				$lastname = $array[2];
				break;
			default:
				$firstname = '';
				$lastname = $name;
				endswitch;

		$address = new stdClass();
		$address->address_user_id = $userid;
		$address->address_firstname = $firstname;
		$address->address_lastname = $lastname;
		$address->address_street = (string)$orderobject->ProcessedOrder->ShippingAddress->AddressFieldOne;
		$address->address_post_code = (string)$orderobject->ProcessedOrder->ShippingAddress->PostalCode;
		$address->address_city = (string)$orderobject->ProcessedOrder->ShippingAddress->City;
		$address->address_telephone = (string)$orderobject->ProcessedOrder->ProcessedOrderItems->ProcessedOrderItem->CartCustomData->TelephonCustomer;
		$address->address_telephone2 = (string)$orderobject->ProcessedOrder->ProcessedOrderItems->ProcessedOrderItem->CartCustomData->TelephonCustomer2;
		$address->address_state = $state;
		$address->address_country = $country;
		$address->address_published = 0;
		
		$addressClass = hikashop_get('class.address');
		$addressid = $addressClass->save($address);
		
		return $addressid;
	}
}